@@ -17,7 +17,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.04';
+$VERSION = '1.07';
 
 my %unescapeVCard = ( '\\'=>'\\', ','=>',', 'n'=>"\n", 'N'=>"\n" );
 
@@ -88,7 +88,10 @@ my %timeInfo = (
 
 %Image::ExifTool::VCard::VCalendar = (
     GROUPS => { 1 => 'VCalendar', 2 => 'Document' },
-    VARS => { NO_LOOKUP => 1 }, # omit tags from lookup
+    VARS => {
+        NO_LOOKUP => 1, # omit tags from lookup
+        LONG_TAGS => 6, # some X-microsoft tags have unavoidably long ID's
+    },
     NOTES => q{
         The VCard module is also used to process iCalendar ICS files since they use
         a format similar to vCard.  The following table lists standard iCalendar
@@ -149,6 +152,51 @@ my %timeInfo = (
     Sequence    => 'SequenceNumber',
     'Request-status' => 'RequestStatus',
     Acknowledged=> { Name => 'Acknowledged',        Groups => { 2 => 'Time' }, %timeInfo },
+#
+# Observed X-tags (not a comprehensive list):
+#
+    'X-apple-calendar-color'=> 'CalendarColor',
+    'X-apple-default-alarm' => 'DefaultAlarm',
+    'X-apple-local-default-alarm' => 'LocalDefaultAlarm',
+    'X-microsoft-cdo-appt-sequence'     => 'AppointmentSequence',
+    'X-microsoft-cdo-ownerapptid'       => 'OwnerAppointmentID',
+    'X-microsoft-cdo-busystatus'        => 'BusyStatus',
+    'X-microsoft-cdo-intendedstatus'    => 'IntendedBusyStatus',
+    'X-microsoft-cdo-alldayevent'       => 'AllDayEvent',
+    'X-microsoft-cdo-importance' => {
+        Name => 'Importance',
+        PrintConv => {
+            0 => 'Low',
+            1 => 'Normal',
+            2 => 'High',
+        },
+    },
+    'X-microsoft-cdo-insttype' => {
+        Name => 'InstanceType',
+        PrintConv => {
+            0 => 'Non-recurring Appointment',
+            1 => 'Recurring Appointment',
+            2 => 'Single Instance of Recurring Appointment',
+            3 => 'Exception to Recurring Appointment',
+        },
+    },
+    'X-microsoft-donotforwardmeeting'   => 'DoNotForwardMeeting',
+    'X-microsoft-disallow-counter'      => 'DisallowCounterProposal',
+    'X-microsoft-locations' => { Name => 'MeetingLocations', Groups => { 2 => 'Location' } },
+    'X-wr-caldesc'          => 'CalendarDescription',
+    'X-wr-calname'          => 'CalendarName',
+    'X-wr-relcalid'         => 'CalendarID',
+    'X-wr-timezone'         => { Name => 'TimeZone2', Groups => { 2 => 'Time' } },
+    'X-wr-alarmuid'         => 'AlarmUID',
+);
+
+%Image::ExifTool::VCard::VNote = (
+    GROUPS => { 1 => 'VNote', 2 => 'Document' },
+    NOTES => 'Tags extracted from V-Note VNT files.',
+    Version => { },
+    Body    => { },
+    Dcreated        => { Name => 'CreateDate', Groups => { 2 => 'Time' }, %timeInfo },
+    'Last-modified' => { Name => 'ModifyDate', Groups => { 2 => 'Time' }, %timeInfo },
 );
 
 #------------------------------------------------------------------------------
@@ -193,14 +241,13 @@ sub DecodeVCardText($$;$)
             $val =~ s/=([0-9a-f]{2})/chr(hex($1))/ige;
         }
         $val = $et->Decode($val, 'UTF8');   # convert from UTF-8
-        # split into separate items if it contains an unescaped comma
-        my $list = $val =~ s/(^|[^\\])((\\\\)*),/$1$2\0/g;
+        # convert unescaped commas to nulls to separate list items
+        $val =~ s/(\\.)|(,)/$1 || "\0"/sge;
         # unescape necessary characters in value
         $val =~ s/\\(.)/$unescapeVCard{$1}||$1/sge;
-        if ($list) {
-            my @vals = split /\0/, $val;
-            $val = \@vals;
-        }
+        # split into list if necessary
+        my @vals = split /\0/, $val;
+        $val = \@vals if @vals > 1;
     }
     return $val;
 }
@@ -216,9 +263,14 @@ sub ProcessVCard($$)
     my $raf = $$dirInfo{RAF};
     my ($buff, $val, $ok, $component, %compNum, @count);
 
-    return 0 unless $raf->Read($buff, 24) and $raf->Seek(0,0) and $buff=~/^BEGIN:(VCARD|VCALENDAR)\r\n/i;
-    my ($type, $lbl, $tbl, $ext) = uc($1) eq 'VCARD' ? qw(VCard vCard Main VCF) : qw(ICS iCalendar VCalendar ICS);
-    $et->SetFileType($type, undef, $ext);
+    return 0 unless $raf->Read($buff, 24) and $raf->Seek(0,0) and $buff=~/^BEGIN:(VCARD|VCALENDAR|VNOTE)\r\n/i;
+    my %info = (
+        VCARD     => [ qw(VCard vCard Main VCF) ],
+        VCALENDAR => [ qw(ICS iCalendar VCalendar ICS) ],
+        VNOTE     => [ qw(VNote vNote VNote VNT text/v-note) ],
+    );
+    my ($type, $lbl, $tbl, $ext, $mime) = @{$info{uc($1)}};
+    $et->SetFileType($type, $mime, $ext);
     return 1 if $$et{OPTIONS}{FastScan} and $$et{OPTIONS}{FastScan} == 3;
     local $/ = "\r\n";
     my $tagTablePtr = GetTagTable("Image::ExifTool::VCard::$tbl");
@@ -236,7 +288,7 @@ sub ProcessVCard($$)
         }
         if ($val =~ /^(BEGIN|END):(V?)(\w+)$/i) {
             my ($begin, $v, $what) = ((lc($1) eq 'begin' ? 1 : 0), $2, ucfirst lc $3);
-            if ($what eq 'Card' or $what eq 'Calendar') {
+            if ($what eq 'Card' or $what eq 'Calendar' or $what eq 'Note') {
                 if ($begin) {
                     @count = ( { } );   # reset group counters
                 } else {
@@ -273,7 +325,7 @@ sub ProcessVCard($$)
             $$et{DOC_NUM} = ++$$et{DOC_COUNT};  # read next card as a new document
         }
         unless ($val =~ s/^([-A-Za-z0-9.]+)//) {
-            $et->WarnOnce("Unrecognized line in $lbl file");
+            $et->Warn("Unrecognized line in $lbl file");
             next;
         }
         my $tag = $1;
@@ -285,7 +337,7 @@ sub ProcessVCard($$)
         } else {
             delete $$et{SET_GROUP1};
         }
-        my ($name, %param, $p, @val);
+        my ($name, %param, $p);
         # vCard tag ID's are case-insensitive, so normalize to lowercase with
         # an uppercase first letter for use as a tag name
         $name = ucfirst $tag if $tag =~ /[a-z]/;    # preserve mixed case in name if it exists
@@ -327,7 +379,7 @@ sub ProcessVCard($$)
                 $param{$p} = '';
             }
         }
-        $val =~ s/^:// or $et->WarnOnce("Invalid line in $lbl file"), next;
+        $val =~ s/^:// or $et->Warn("Invalid line in $lbl file"), next;
         # add 'Type' parameter to id and name if it exists
         $param{Type} and $tag .= $param{Type}, $name .= $param{Type};
         # convert base64-encoded data
@@ -378,7 +430,7 @@ information from vCard VCF and iCalendar ICS files.
 
 =head1 AUTHOR
 
-Copyright 2003-2017, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2025, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
